@@ -88,7 +88,7 @@ import Network.OAuth.Http.PercentEncoding
 import Control.Monad
 import Control.Monad.Trans
 import System.IO
-import System.Random (randomRIO)
+import System.Entropy (getEntropy)
 import System.Locale (defaultTimeLocale)
 import Data.Time (getCurrentTime,formatTime)
 import Data.Char (chr,ord)
@@ -97,12 +97,14 @@ import Data.Word (Word8)
 import qualified Data.Binary as Bi
 import qualified Data.Digest.Pure.SHA as S
 import qualified Codec.Binary.Base64 as B64
+import qualified Data.ByteString as BS
 import qualified Data.ByteString.Lazy as B
-import qualified Codec.Crypto.RSA as R
+import qualified Codec.Crypto.RSA        as R
+import qualified Crypto.Types.PubKey.RSA as R
 
 -- | A request that is ready to be performed, i.e., that contains authorization headers.
 newtype OAuthRequest = OAuthRequest { unpackRq :: Request }
-                     deriving (Show,Eq)
+                     deriving (Show)
 
 -- | Random string that is unique amongst requests. Refer to <http://oauth.net/core/1.0/#nonce> for more information.
 newtype Nonce = Nonce { unNonce :: String }
@@ -202,9 +204,13 @@ signature m token req = case m
                                             ]
 
         params = if (ifindWithDefault ("content-type","") (reqHeaders req) == "application/x-www-form-urlencoded")
+
+                 -- e.g., in the case of most Twitter API calls
                  then (qString req) `unionAll` (parseQString . map (chr.fromIntegral) 
                                                              . B.unpack 
                                                              . reqPayload $ req)
+
+                 -- e.g., in the case of a "multipart/form-data" image upload, however, the payload isn't signed
                  else qString req
 
 -- | Returns true if the token is able to perform 2-legged oauth requests.
@@ -368,9 +374,7 @@ authorization m realm nonce time token req = oauthPrefix ++ enquote (("oauth_sig
         enquote = intercalate "," . map (\(k,v) -> encode k ++"=\""++ encode v ++"\"")
 
 _nonce :: (MonadIO m) => m Nonce
-_nonce = do { rand <- liftIO (randomRIO (0,maxBound::Int))
-            ; return (Nonce $ show rand)
-            }
+_nonce = liftIO $ liftM (Nonce . B64.encode . BS.unpack) (getEntropy 32)
 
 _timestamp :: (MonadIO m) => m Timestamp
 _timestamp = do { clock <- liftIO getCurrentTime
